@@ -74,6 +74,7 @@ where
         session_key: ESK,
         circuit_pub_keys: Vec<<ESK as DiffieHellman>::PK>,
         routing_info: &[A],
+        dest: impl Address,
         max_relays: usize,
         payload: &[u8],
     ) -> Result<(Vec<<ESK as DiffieHellman>::SSK>, Self), SfynxError> {
@@ -84,6 +85,7 @@ where
             session_key,
             circuit_pub_keys,
             routing_info,
+            dest,
             max_relays,
             payload,
             &shared_secrets,
@@ -94,6 +96,7 @@ where
         session_key: ESK,
         circuit_pub_keys: Vec<<ESK as DiffieHellman>::PK>,
         routing_info: &[A],
+        dest: impl Address,
         max_relays: usize,
         payload: &[u8],
         shared_secrets: &[<ESK as DiffieHellman>::SSK],
@@ -105,6 +108,7 @@ where
         let (_, header) = Header::<A, HMAC, SC, ESK, H>::with_shared_secrets(
             max_relays,
             routing_info,
+            dest,
             session_key,
             shared_secrets,
         )?;
@@ -154,15 +158,21 @@ where
         Ok(encrypted_payload)
     }
 
-    fn decrypt_payload(&self, secret: &<ESK as DiffieHellman>::SSK) -> Result<Vec<u8>, SfynxError> {
-        let mut decrypted_payload = self.payload.clone();
+    /// Decrypt payload in-place with provided `secret` key.
+    ///
+    /// # Safety
+    /// Unsafe because it ignores header integrity.
+    /// Should be used with caution.
+    pub unsafe fn decrypt_payload(
+        &self,
+        payload: &mut [u8],
+        secret: &<ESK as DiffieHellman>::SSK,
+    ) -> Result<(), SfynxError> {
+        let cipher = generate_cipher_stream::<SC>(&secret.to_vec(), &[0; 12], payload.len())
+            .map_err(|e| SfynxError::StreamCipherError(format!("{:?}", e)))?;
+        xor(payload, &cipher);
 
-        let cipher =
-            generate_cipher_stream::<SC>(&secret.to_vec(), &[0; 12], decrypted_payload.len())
-                .map_err(|e| SfynxError::StreamCipherError(format!("{:?}", e)))?;
-        xor(&mut decrypted_payload, &cipher);
-
-        Ok(decrypted_payload)
+        Ok(())
     }
 
     /// Wrap payload in an onion encryption layer in-place.
@@ -185,7 +195,8 @@ where
         let shared_secret = session_key.diffie_hellman(&self.header.public_key);
         let (next_addr, header) = self.header.peel(&shared_secret)?;
 
-        let payload = self.decrypt_payload(&shared_secret)?;
+        let mut payload = self.payload.clone();
+        unsafe { self.decrypt_payload(&mut payload, &shared_secret)? };
 
         Ok((
             shared_secret,
